"""CLI to test all API functions and models.

Execute with:
poetry run python3 -m myskoda.cli
"""

import asyncclick as click
from aiohttp import ClientSession
from asyncclick.core import Context
from termcolor import colored

from . import idk_authorize
from .models.charging import MaxChargeCurrent
from .models.common import (
    ActiveState,
    ChargerLockedState,
    ConnectionState,
    DoorLockedState,
    OnOffState,
    OpenState,
)
from .rest_api import RestApi


@click.group()
@click.version_option()
@click.option("username", "--user", help="Username used for login.", required=True)
@click.option("password", "--password", help="Password used for login.", required=True)
@click.pass_context
def cli(ctx: Context, username: str, password: str) -> None:
    """Interact with the MySkoda API."""
    ctx.ensure_object(dict)
    ctx.obj["username"] = username
    ctx.obj["password"] = password


@cli.command()
@click.pass_context
async def auth(ctx: Context) -> None:
    """Extract the auth token."""
    async with ClientSession() as session:
        auth_codes = await idk_authorize(session, ctx.obj["username"], ctx.obj["password"])
        print(auth_codes.access_token)


@cli.command()
@click.pass_context
async def list_vehicles(ctx: Context) -> None:
    """Print a list of all vehicle identification numbers associated with the account."""
    async with ClientSession() as session:
        hub = RestApi(session)
        await hub.authenticate(ctx.obj["username"], ctx.obj["password"])
        print(f"{colored("vehicles:", "blue")}")
        for vehicle in await hub.list_vehicles():
            print(f"- {vehicle}")


@cli.command()
@click.argument("vin")
@click.pass_context
async def info(ctx: Context, vin: str) -> None:
    """Print info for the specified vin."""
    async with ClientSession() as session:
        hub = RestApi(session)
        await hub.authenticate(ctx.obj["username"], ctx.obj["password"])
        info = await hub.get_info(vin)

        if info.specification.battery is not None:
            print(
                f"{colored("battery capacity:", "blue")} {info.specification.battery.capacity}kwh"
            )
        print(f"{colored("power:", "blue")} {info.specification.engine.power}kw")
        print(f"{colored("engine:", "blue")} {info.specification.engine.type}")
        print(f"{colored("model:", "blue")} {info.specification.model}")
        print(f"{colored("model id:", "blue")} {info.specification.system_model_id}")
        print(f"{colored("model year:", "blue")} {info.specification.model_year}")
        print(f"{colored("title:", "blue")} {info.specification.title}")
        print(f"{colored("vin:", "blue")} {info.vin}")
        print(f"{colored("software:", "blue")} {info.software_version}")


@cli.command()
@click.argument("vin")
@click.pass_context
async def status(ctx: Context, vin: str) -> None:
    """Print current status for the specified vin."""
    async with ClientSession() as session:
        hub = RestApi(session)
        await hub.authenticate(ctx.obj["username"], ctx.obj["password"])
        status = await hub.get_status(vin)

        print(
            f"{colored("doors:", "blue")} {c_open(status.overall.doors)}, "
            f"{locked(status.overall.doors_locked)}"
        )
        print(f"{colored("bonnet:", "blue")} {c_open(status.detail.bonnet)}")
        print(f"{colored("trunk:", "blue")} {c_open(status.detail.trunk)}")
        print(f"{colored("windows:", "blue")} {c_open(status.overall.windows)}")
        print(f"{colored("lights:", "blue")} {on(status.overall.lights)}")
        print(f"{colored("last update:", "blue")} {status.car_captured_timestamp}")


@cli.command()
@click.argument("vin")
@click.pass_context
async def air_conditioning(ctx: Context, vin: str) -> None:
    """Print current status about air conditioning."""
    async with ClientSession() as session:
        hub = RestApi(session)
        await hub.authenticate(ctx.obj["username"], ctx.obj["password"])
        ac = await hub.get_air_conditioning(vin)

        print(f"{colored("window heating:", "blue")} {bool_state(ac.window_heating_enabled)}")
        print(f"{colored("window heating (front):", "blue")} {on(ac.window_heating_state.front)}")
        print(f"{colored("window heating (back):", "blue")} {on(ac.window_heating_state.rear)}")
        if ac.target_temperature is not None:
            print(
                f"{colored("target temperature:", "blue")} "
                f"{ac.target_temperature.temperature_value}°C"
            )
        print(f"{colored("steering wheel position:", "blue")} {ac.steering_wheel_position}")
        print(f"{colored("air conditioning:", "blue")} {on(ac.state)}")
        print(f"{colored("state:", "blue")} {ac.state}")
        print(
            f"{colored("charger:", "blue")} {connected(ac.charger_connection_state)}, "
            f"{charger_locked(ac.charger_lock_state)}"
        )
        print(
            f"{colored("temperature reached:", "blue")} "
            f"{ac.estimated_date_time_to_reach_target_temperature}"
        )


@cli.command()
@click.argument("vin")
@click.pass_context
async def positions(ctx: Context, vin: str) -> None:
    """Print the vehicle's current position."""
    async with ClientSession() as session:
        hub = RestApi(session)
        await hub.authenticate(ctx.obj["username"], ctx.obj["password"])
        positions = await hub.get_positions(vin)

        for position in positions.positions:
            print(f"- {colored("type:", "blue")} {position.type}")
            print(f"  {colored("latitude:", "blue")} {position.gps_coordinates.latitude}")
            print(f"  {colored("longitude:", "blue")} {position.gps_coordinates.longitude}")
            print(f"  {colored("address:", "blue")}")
            print(f"     {position.address.street} {position.address.house_number}")
            print(f"     {position.address.zip_code} {position.address.city}")
            print(f"     {position.address.country} ({position.address.country_code})")

        if positions.errors:
            print(f"{colored("Error:", "red")}")
        for error in positions.errors:
            print(f"- {colored("type:", "blue")} {error.type}")
            print(f"  {colored("description:", "blue")} {error.description}")


@cli.command()
@click.argument("vin")
@click.pass_context
async def health(ctx: Context, vin: str) -> None:
    """Print the vehicle's mileage."""
    async with ClientSession() as session:
        hub = RestApi(session)
        await hub.authenticate(ctx.obj["username"], ctx.obj["password"])
        health = await hub.get_health(vin)

        print(f"{colored("mileage:", "blue")} {health.mileage_in_km}km")
        print(f"{colored("last updated:", "blue")} {health.captured_at}")


@cli.command()
@click.argument("vin")
@click.pass_context
async def charging(ctx: Context, vin: str) -> None:
    """Print the vehicle's current charging state."""
    async with ClientSession() as session:
        hub = RestApi(session)
        await hub.authenticate(ctx.obj["username"], ctx.obj["password"])
        charging = await hub.get_charging(vin)

        if charging.status is not None:
            print(
                f"{colored("battery charge:", "blue")} "
                f"{charging.status.battery.state_of_charge_in_percent}%"
            )
            print(
                f"{colored("remaining distance:", "blue")} "
                f"{charging.status.battery.remaining_cruising_range_in_meters / 1000}km"
            )
            print(f"{colored("state:", "blue")} {charging.status.state}")
            print(f"{colored("charger type:", "blue")} {charging.status.charge_type}")
            print(
                f"{colored("charging rate:", "blue")} "
                f"{charging.status.charging_rate_in_kilometers_per_hour}km/h"
            )
            print(
                f"{colored("remaining time:", "blue")} "
                f"{charging.status.remaining_time_to_fully_charged_in_minutes}min"
            )
            print(f"{colored("charging power:", "blue")} {charging.status.charge_power_in_kw}kw")
        print(
            f"{colored("target:", "blue")} {charging.settings.target_state_of_charge_in_percent}%"
        )
        print(
            f"{colored("battery care mode:", "blue")} "
            f"{active(charging.settings.charging_care_mode)}"
        )
        print(
            f"{colored("current:", "blue")} "
            f"{charge_current(charging.settings.max_charge_current_ac)}"
        )


@cli.command()
@click.argument("vin")
@click.pass_context
async def maintenance(ctx: Context, vin: str) -> None:
    """Print the vehicle's maintenance information."""
    async with ClientSession() as session:
        hub = RestApi(session)
        await hub.authenticate(ctx.obj["username"], ctx.obj["password"])
        maintenance = await hub.get_maintenance(vin)

        print(f"{colored("mileage:", "blue")} {maintenance.maintenance_report.mileage_in_km}km")
        print(
            f"{colored("inspection due in:", "blue")} "
            f"{maintenance.maintenance_report.inspection_due_in_days}d / "
            f"{maintenance.maintenance_report.inspection_due_in_km}km"
        )
        print(
            f"{colored("oil service due in:", "blue")} "
            f"{maintenance.maintenance_report.oil_service_due_in_days}d / "
            f"{maintenance.maintenance_report.oil_service_due_in_km}km"
        )
        print(f"{colored("service partner:", "blue")} {maintenance.preferred_service_partner.name}")
        address = maintenance.preferred_service_partner.address
        print(f"     {address.street} {address.house_number}")
        print(f"     {address.zip_code} {address.city}")
        print(f"     {address.country} ({address.country_code})")
        if maintenance.predictive_maintenance:
            print(f"{colored("email:", "blue")} {maintenance.predictive_maintenance.setting.email}")
            print(f"{colored("phone:", "blue")} {maintenance.predictive_maintenance.setting.phone}")


@cli.command()
@click.argument("vin")
@click.pass_context
async def driving_range(ctx: Context, vin: str) -> None:
    """Print the vehicle's estimated driving range information."""
    async with ClientSession() as session:
        hub = RestApi(session)
        await hub.authenticate(ctx.obj["username"], ctx.obj["password"])
        driving_range = await hub.get_driving_range(vin)

        print(f"{colored("range:", "blue")} {driving_range.total_range_in_km}km")
        print(f"{colored("car type:", "blue")} {driving_range.car_type}")
        print(f"{colored("last update:", "blue")} {driving_range.car_captured_timestamp}")
        print(
            f"{colored("fuel level:", "blue")} "
            f"{driving_range.primary_engine_range.current_fuel_level_in_percent}%"
        )


@cli.command()
@click.argument("vin")
@click.pass_context
async def trip_statistics(ctx: Context, vin: str) -> None:
    """Print the last trip statics."""
    async with ClientSession() as session:
        hub = RestApi(session)
        await hub.authenticate(ctx.obj["username"], ctx.obj["password"])
        stats = await hub.get_trip_statistics(vin)

        print(
            f"{colored("overall fuel consumption:", "blue")} "
            f"{stats.overall_average_fuel_consumption}l/100km"
        )
        print(f"{colored("mileage:", "blue")} {stats.overall_mileage_in_km}km")
        print(f"{colored("entries:", "blue")}")
        for entry in stats.detailed_statistics:
            print(f"- {colored("date:", "blue")} {entry.date}")
            print(
                f"  {colored("average fuel consumption:", "blue")} "
                f"{entry.average_fuel_consumption}l/100km"
            )
            print(f"  {colored("average speed:", "blue")} {entry.average_speed_in_kmph}km/h")
            print(f"  {colored("mileage:", "blue")} {entry.mileage_in_km}km")


def c_open(cond: OpenState) -> str:
    return colored("open", "red") if cond == OpenState.OPEN else colored("closed", "green")


def locked(cond: DoorLockedState) -> str:
    return (
        colored("locked", "green") if cond == DoorLockedState.LOCKED else colored("unlocked", "red")
    )


def on(cond: OnOffState) -> str:
    return colored("on", "green") if cond == OnOffState.ON else colored("off", "red")


def connected(cond: ConnectionState) -> str:
    return (
        colored("connected", "green")
        if cond == ConnectionState.CONNECTED
        else colored("disconnected", "red")
    )


def active(cond: ActiveState) -> str:
    return (
        colored("active", "green") if cond == ActiveState.ACTIVATED else colored("inactive", "red")
    )


def charge_current(cond: MaxChargeCurrent) -> str:
    return (
        colored("maximum", "green")
        if cond == MaxChargeCurrent.MAXIMUM
        else colored("reduced", "red")
    )


def bool_state(cond: bool) -> str:  # noqa: FBT001
    return colored("true", "green") if cond else colored("false", "red")


def charger_locked(cond: ChargerLockedState) -> str:
    return (
        colored("locked", "green")
        if cond == ChargerLockedState.LOCKED
        else colored("unlocked", "red")
    )


if __name__ == "__main__":
    cli()



"""Contains API representation for the MySkoda REST API."""

import logging
from asyncio import gather

from aiohttp import ClientSession

from .authorization import IDKSession, idk_authorize
from .const import BASE_URL_SKODA
from .models.air_conditioning import AirConditioning
from .models.charging import Charging
from .models.driving_range import DrivingRange
from .models.health import Health
from .models.info import Info
from .models.maintenance import Maintenance
from .models.position import Positions, PositionType
from .models.status import Status
from .models.trip_statistics import TripStatistics

_LOGGER = logging.getLogger(__name__)


class Vehicle:
    """Wrapper class for all information from all endpoints."""

    info: Info
    charging: Charging
    status: Status
    air_conditioning: AirConditioning
    position: Positions
    health: Health

    def __init__(  # noqa: D107, PLR0913
        self,
        info: Info,
        charging: Charging,
        status: Status,
        air_conditioning: AirConditioning,
        position: Positions,
        health: Health,
    ) -> None:
        self.info = info
        self.charging = charging
        self.status = status
        self.air_conditioning = air_conditioning
        self.position = position
        self.health = health


class RestApi:
    """API hub class that can perform all calls to the MySkoda API."""

    session: ClientSession
    idk_session: IDKSession

    def __init__(self, session: ClientSession) -> None:  # noqa: D107
        self.session = session

    async def authenticate(self, email: str, password: str) -> bool:
        """Perform the full login process.

        Must be called before any other methods on the class can be called.
        """
        self.idk_session = await idk_authorize(self.session, email, password)

        _LOGGER.info("IDK Authorization was successful.")

        return True

    async def get_info(self, vin: str) -> Info:
        """Retrieve the basic vehicle information for the specified vehicle."""
        async with self.session.get(
            f"{BASE_URL_SKODA}/api/v2/garage/vehicles/{vin}?connectivityGenerations=MOD1&connectivityGenerations=MOD2&connectivityGenerations=MOD3&connectivityGenerations=MOD4",
            headers=await self._headers(),
        ) as response:
            _LOGGER.debug("vin %s: Received basic info", vin)
            return Info(**await response.json())

    async def get_charging(self, vin: str) -> Charging:
        """Retrieve information related to charging for the specified vehicle."""
        async with self.session.get(
            f"{BASE_URL_SKODA}/api/v1/charging/{vin}", headers=await self._headers()
        ) as response:
            _LOGGER.debug("Received charging info")
            return Charging(**await response.json())

    async def get_status(self, vin: str) -> Status:
        """Retrieve the current status for the specified vehicle."""
        async with self.session.get(
            f"{BASE_URL_SKODA}/api/v2/vehicle-status/{vin}",
            headers=await self._headers(),
        ) as response:
            _LOGGER.debug("vin %s: Received status")
            return Status(**await response.json())

    async def get_air_conditioning(self, vin: str) -> AirConditioning:
        """Retrieve the current air conditioning status for the specified vehicle."""
        async with self.session.get(
            f"{BASE_URL_SKODA}/api/v2/air-conditioning/{vin}",
            headers=await self._headers(),
        ) as response:
            _LOGGER.debug("vin %s: Received air conditioning")
            return AirConditioning(**await response.json())

    async def get_positions(self, vin: str) -> Positions:
        """Retrieve the current position for the specified vehicle."""
        async with self.session.get(
            f"{BASE_URL_SKODA}/api/v1/maps/positions?vin={vin}",
            headers=await self._headers(),
        ) as response:
            _LOGGER.debug("vin %s: Received position")
            return Positions(**await response.json())

    async def get_driving_range(self, vin: str) -> DrivingRange:
        """Retrieve estimated driving range for combustion vehicles."""
        async with self.session.get(
            f"{BASE_URL_SKODA}/api/v2/vehicle-status/{vin}/driving-range",
            headers=await self._headers(),
        ) as response:
            _LOGGER.debug("vin %s: Received driving range")
            return DrivingRange(**await response.json())

    async def get_trip_statistics(self, vin: str) -> TripStatistics:
        """Retrieve statistics about past trips."""
        async with self.session.get(
            f"{BASE_URL_SKODA}/api/v1/trip-statistics/{vin}?offsetType=week&offset=0&timezone=Europe%2FBerlin",
            headers=await self._headers(),
        ) as response:
            _LOGGER.debug("vin %s: Received trip statistics")
            return TripStatistics(**await response.json())

    async def get_maintenance(self, vin: str) -> Maintenance:
        """Retrieve maintenance report."""
        async with self.session.get(
            f"{BASE_URL_SKODA}/api/v3/vehicle-maintenance/vehicles/{vin}",
            headers=await self._headers(),
        ) as response:
            _LOGGER.debug("vin %s: Received maintenance report")
            return Maintenance(**await response.json())

    async def get_health(self, vin: str) -> Health:
        """Retrieve health information for the specified vehicle."""
        async with self.session.get(
            f"{BASE_URL_SKODA}/api/v1/vehicle-health-report/warning-lights/{vin}",
            headers=await self._headers(),
        ) as response:
            _LOGGER.debug("vin %s: Received health")
            return Health(**await response.json())

    async def list_vehicles(self) -> list[str]:
        """List all vehicles by their vins."""
        async with self.session.get(
            f"{BASE_URL_SKODA}/api/v2/garage?connectivityGenerations=MOD1&connectivityGenerations=MOD2&connectivityGenerations=MOD3&connectivityGenerations=MOD4",
            headers=await self._headers(),
        ) as response:
            json = await response.json()
            return [vehicle["vin"] for vehicle in json["vehicles"]]

    async def get_vehicle(self, vin: str) -> Vehicle:
        """Retrieve all information about a given vehicle by calling all endpoints."""
        [info, charging, status, air_conditioning, position, health] = await gather(
            *[
                self.get_info(vin),
                self.get_charging(vin),
                self.get_status(vin),
                self.get_air_conditioning(vin),
                self.get_positions(vin),
                self.get_health(vin),
            ]
        )
        return Vehicle(
            info=info,
            charging=charging,
            status=status,
            air_conditioning=air_conditioning,
            position=position,
            health=health,
        )

    async def get_all_vehicles(self) -> list[Vehicle]:
        """Call all endpoints for all vehicles in the user's garage."""
        return await gather(*[self.get_vehicle(vehicle) for vehicle in await self.list_vehicles()])

    async def _headers(self) -> dict[str, str]:
        return {"authorization": f"Bearer {await self.idk_session.get_access_token(self.session)}"}

    async def stop_air_conditioning(self, vin: str) -> None:
        """Stop the air conditioning."""
        async with self.session.post(
            f"{BASE_URL_SKODA}/api/v2/air-conditioning/{vin}/stop",
            headers=await self._headers(),
        ) as response:
            await response.text()

    async def start_air_conditioning(self, vin: str, temperature: str) -> None:
        """Start the air conditioning."""
        json_data = {
            "heaterSource": "ELECTRIC",
            "targetTemperature": {
                "temperatureValue": temperature,
                "unitInCar": "CELSIUS",
            },
        }
        async with self.session.post(
            f"{BASE_URL_SKODA}/api/v2/air-conditioning/{vin}/start",
            headers=await self._headers(),
            json=json_data,
        ) as response:
            await response.text()

    async def set_target_temperature(self, vin: str, temperature: str) -> None:
        """Set the air conditioning's target temperature in °C."""
        json_data = {"temperatureValue": temperature, "unitInCar": "CELSIUS"}
        async with self.session.post(
            f"{BASE_URL_SKODA}/api/v2/air-conditioning/{vin}/settings/target-temperature",
            headers=await self._headers(),
            json=json_data,
        ) as response:
            await response.text()

    async def start_window_heating(self, vin: str) -> None:
        """Start heating both the front and rear window."""
        async with self.session.post(
            f"{BASE_URL_SKODA}/api/v2/air-conditioning/{vin}/start-window-heating",
            headers=await self._headers(),
        ) as response:
            await response.text()

    async def stop_window_heating(self, vin: str) -> None:
        """Stop heating both the front and rear window."""
        async with self.session.post(
            f"{BASE_URL_SKODA}/api/v2/air-conditioning/{vin}/stop-window-heating",
            headers=await self._headers(),
        ) as response:
            await response.text()

    async def set_charge_limit(self, vin: str, limit: int) -> None:
        """Set the maximum charge limit in percent."""
        json_data = {"targetSOCInPercent": limit}
        async with self.session.put(
            f"{BASE_URL_SKODA}/api/v1/charging/{vin}/set-charge-limit",
            headers=await self._headers(),
            json=json_data,
        ) as response:
            await response.text()

    # TODO @dvx76: Maybe refactor for FBT001
    async def set_battery_care_mode(self, vin: str, enabled: bool) -> None:  # noqa: FBT001
        """Enable or disable the battery care mode."""
        json_data = {"chargingCareMode": "ACTIVATED" if enabled else "DEACTIVATED"}
        async with self.session.put(
            f"{BASE_URL_SKODA}/api/v1/charging/{vin}/set-care-mode",
            headers=await self._headers(),
            json=json_data,
        ) as response:
            await response.text()

    # TODO @dvx76: Maybe refactor for FBT001
    async def set_reduced_current_limit(self, vin: str, reduced: bool) -> None:  # noqa: FBT001
        """Enable reducing the current limit by which the car is charged."""
        json_data = {"chargingCurrent": "REDUCED" if reduced else "MAXIMUM"}
        async with self.session.put(
            f"{BASE_URL_SKODA}/api/v1/charging/{vin}/set-charging-current",
            headers=await self._headers(),
            json=json_data,
        ) as response:
            await response.text()

    async def start_charging(self, vin: str) -> None:
        """Start charging the car."""
        async with self.session.post(
            f"{BASE_URL_SKODA}/api/v1/charging/{vin}/start",
            headers=await self._headers(),
        ) as response:
            await response.text()

    async def stop_charging(self, vin: str) -> None:
        """Stop charging the car."""
        async with self.session.post(
            f"{BASE_URL_SKODA}/api/v1/charging/{vin}/stop",
            headers=await self._headers(),
        ) as response:
            await response.text()

    async def wakeup(self, vin: str) -> None:
        """Wake the vehicle up. Can be called maximum three times a day."""
        async with self.session.post(
            f"{BASE_URL_SKODA}/api/v1/vehicle-wakeup/{vin}?applyRequestLimiter=true",
            headers=await self._headers(),
        ) as response:
            await response.text()

    # TODO @dvx76: Maybe refactor for FBT001
    async def honk_flash(self, vin: str, honk: bool = False) -> None:  # noqa: FBT001, FBT002
        """Honk and/or flash."""
        positions = await self.get_positions(vin)
        position = next(pos for pos in positions.positions if pos.type == PositionType.VEHICLE)
        json_data = {
            "mode": "HONK_AND_FLASH" if honk else "FLASH",
            "vehiclePosition": {
                "lat": position.gps_coordinates.latitude,
                "lng": position.gps_coordinates.longitude,
            },
        }
        async with self.session.post(
            f"{BASE_URL_SKODA}/api/v1/vehicle-access/{vin}/honk-and-flash",
            headers=await self._headers(),
            json=json_data,
        ) as response:
            await response.text()



"""Models for responses of api/v1/charging endpoint."""

from datetime import datetime
from enum import StrEnum
from typing import Any

from pydantic import BaseModel, Field

from .common import ActiveState, EnabledState


class ChargeMode(StrEnum):
    MANUAL = "MANUAL"


class MaxChargeCurrent(StrEnum):
    MAXIMUM = "MAXIMUM"
    REDUCED = "REDUCED"


class ChargingState(StrEnum):
    READY_FOR_CHARGING = "READY_FOR_CHARGING"
    CONNECT_CABLE = "CONNECT_CABLE"
    CONSERVING = "CONSERVING"
    CHARGING = "CHARGING"


class ChargeType(StrEnum):
    AC = "AC"
    DC = "DC"


class PlugUnlockMode(StrEnum):
    PERMANENT = "PERMANENT"
    ON = "ON"
    OFF = "OFF"


class Settings(BaseModel):
    available_charge_modes: list[ChargeMode] = Field(None, alias="availableChargeModes")
    battery_support: EnabledState = Field(None, alias="batterySupport")
    charging_care_mode: ActiveState = Field(None, alias="chargingCareMode")
    max_charge_current_ac: MaxChargeCurrent = Field(None, alias="maxChargeCurrentAc")
    preferred_charge_mode: ChargeMode = Field(None, alias="preferredChargeMode")
    target_state_of_charge_in_percent: int = Field(None, alias="targetStateOfChargeInPercent")
    auto_unlock_plug_when_charged: PlugUnlockMode = Field(None, alias="autoUnlockPlugWhenCharged")


class Battery(BaseModel):
    remaining_cruising_range_in_meters: int = Field(None, alias="remainingCruisingRangeInMeters")
    state_of_charge_in_percent: int = Field(None, alias="stateOfChargeInPercent")


class Status(BaseModel):
    battery: Battery
    charge_power_in_kw: float | None = Field(None, alias="chargePowerInKw")
    charging_rate_in_kilometers_per_hour: float = Field(
        None, alias="chargingRateInKilometersPerHour"
    )
    remaining_time_to_fully_charged_in_minutes: int = Field(
        None, alias="remainingTimeToFullyChargedInMinutes"
    )
    charge_type: ChargeType | None = Field(None, alias="chargeType")
    state: ChargingState


class Charging(BaseModel):
    """Information related to charging an EV."""

    car_captured_timestamp: datetime = Field(None, alias="carCapturedTimestamp")
    errors: list[Any]
    is_vehicle_in_saved_location: bool = Field(None, alias="isVehicleInSavedLocation")
    settings: Settings
    status: Status | None



"""Handles authorization to the MySkoda API."""

import base64
import hashlib
import json
import logging
import random
import re
import string
import uuid
from datetime import UTC, datetime, timedelta
from typing import cast

import jwt
import yaml
from aiohttp import ClientSession, FormData
from bs4 import BeautifulSoup
from pydantic import BaseModel, Field

from .const import BASE_URL_IDENT, BASE_URL_SKODA, CLIENT_ID

_LOGGER = logging.getLogger(__name__)


class IDKCredentials:
    """Stores the credentials that are used to perform the login."""

    hmac: str
    csrf: str
    relay_state: str
    email: str
    password: str

    def __init__(self, data: dict, email: str, password: str) -> None:
        """Create a new set of credentials.

        The dict is supposed to contain the hmac, csrf and relay state from the website.
        """
        self.email = email
        self.password = password
        self.update(data)

    def update(self, data: dict) -> None:
        """Update the values from the website in case the changed."""
        self.csrf = cast(str, data.get("csrf_token"))
        self.hmac = cast(str, data.get("templateModel", {}).get("hmac"))
        self.relay_state = data.get("templateModel", {}).get("relayState")


class IDKAuthorizationCode(BaseModel):
    """One-time authorization code that can be obtained by logging in.

    This authorization code can later be exchanged for a set of JWT tokens.
    """

    code: str
    token_type: str
    id_token: str


class IDKSession(BaseModel):
    """Stores the JWT tokens relevant for a session at the IDK server.

    Can be used to authorized and refresh the authorization token.
    """

    access_token: str = Field(None, alias="accessToken")
    refresh_token: str = Field(None, refreshToken="accessToken")
    id_token: str = Field(None, idToken="accessToken")

    async def perform_refresh(self, session: ClientSession) -> None:
        """Refresh the authorization token.

        This will consume the `refresh_token` and exchange it for a new set of tokens.
        """
        json_data = {"token": self.refresh_token}
        async with session.post(
            f"{BASE_URL_SKODA}/api/v1/authentication/refresh-token?tokenType=CONNECT",
            json=json_data,
        ) as response:
            data = json.loads(await response.text())
            self.access_token = data.get("accessToken")
            self.refresh_token = data.get("refreshToken")
            self.id_token = data.get("idToken")

    async def get_access_token(self, session: ClientSession) -> str:
        """Get the access token.

        Use this method instead of using `access_token` directly. It will automatically
        check if the JWT token is about to expire and refresh it using the `refresh_token`.
        """
        meta = jwt.decode(self.access_token, options={"verify_signature": False})
        expiry = datetime.fromtimestamp(cast(float, meta.get("exp")), tz=UTC)
        if datetime.now(tz=UTC) + timedelta(minutes=10) > expiry:
            _LOGGER.info("Refreshing IDK access token")
            await self.perform_refresh(session)
        return self.access_token


def _extract_states_from_website(html: str) -> dict[str, str]:
    """Information such as the CSRF or the hmac will be available in the HTML.

    This method will parse the information from a `<script>` tag in the HTML using BS4.
    """
    soup = BeautifulSoup(html, "html.parser")

    # Regex to extract the information assigned to `window._IDK` from the script tag.
    json_object = re.compile(r"window\._IDK\s=\s((?:\n|.)*?)$")

    data = None

    # Search through all script tags and find the first one to match the Regex.
    for script in soup.find_all("script"):
        if len(script.contents) != 1:
            continue
        content = script.contents[0].strip()
        result = json_object.search(content)
        if result is None:
            continue

        result = result.group(1)
        # Load the info using YAML, since the syntax used in the script is YAML compatible,
        # but not JSON compatible (missing quotes around field names, trailing commas).
        data = yaml.safe_load(result)

    if data is None:
        raise AuthorizationError

    return data


async def _initial_oidc_authorize(
    session: ClientSession, verifier: str, email: str, password: str
) -> IDKCredentials:
    """First step of the login process.

    This calls the route for initial authorization, which will contain the initial SSO information
    such as the CSRF or the HMAC.
    """
    # A SHA256 hash of the random "verifier" string will be transmitted as a challenge.
    # This is part of the OAUTH2 PKCE process. It is described here in detail:
    # https://www.oauth.com/oauth2-servers/pkce/authorization-request/
    verifier_hash = hashlib.sha256(verifier.encode("utf-8")).digest()
    challenge = (
        base64.b64encode(verifier_hash)
        .decode("utf-8")
        .replace("+", "-")
        .replace("/", "_")
        .rstrip("=")
    )

    params = {
        "client_id": CLIENT_ID,
        "nonce": str(uuid.uuid4()),
        "redirect_uri": "myskoda://redirect/login/",
        "response_type": "code id_token",
        # OpenID scopes. Can be found here: https://identity.vwgroup.io/.well-known/openid-configuration
        "scope": "address badge birthdate cars driversLicense dealers email mileage mbb nationalIdentifier openid phone profession profile vin",  # noqa: E501
        "code_challenge": challenge,
        "code_challenge_method": "s256",
        "prompt": "login",
    }
    async with session.get(f"{BASE_URL_IDENT}/oidc/v1/authorize", params=params) as response:
        data = _extract_states_from_website(await response.text())
        return IDKCredentials(data, email, password)


async def _enter_email_address(
    session: ClientSession, login_meta: IDKCredentials
) -> IDKCredentials:
    """Second step in the login process.

    Will post only the email address to the backend. The password will follow in a later request.
    """
    form_data = FormData()
    form_data.add_field("relayState", login_meta.relay_state)
    form_data.add_field("email", login_meta.email)
    form_data.add_field("hmac", login_meta.hmac)
    form_data.add_field("_csrf", login_meta.csrf)

    async with session.post(
        f"{BASE_URL_IDENT}/signin-service/v1/{CLIENT_ID}/login/identifier",
        data=form_data(),
    ) as response:
        data = _extract_states_from_website(await response.text())
        login_meta.update(data)
        return login_meta


async def _enter_password(
    session: ClientSession, login_meta: IDKCredentials
) -> IDKAuthorizationCode:
    """Third step in the login process.

    Post both the email address and the password to the backend.
    This will return a token which can then be used in the skoda services to authenticate.
    """
    form_data = FormData()
    form_data.add_field("relayState", login_meta.relay_state)
    form_data.add_field("email", login_meta.email)
    form_data.add_field("password", login_meta.password)
    form_data.add_field("hmac", login_meta.hmac)
    form_data.add_field("_csrf", login_meta.csrf)

    # The following is a bit hacky:
    # The backend will redirect multiple times after the login was successful.
    # The last redirect will redirect back to the `MySkoda` app in Android,
    # using the `myskoda://` URL prefix.
    # The following loop will follow all redirects until the last redirect to `myskoda://` is
    # encountered. This last URL will contain the token.
    async with session.post(
        f"{BASE_URL_IDENT}/signin-service/v1/{CLIENT_ID}/login/authenticate",
        data=form_data(),
        allow_redirects=False,
    ) as auth_response:
        location = auth_response.headers["Location"]
        while not location.startswith("myskoda://"):
            async with session.get(location, allow_redirects=False) as response:
                location = response.headers["Location"]
        codes = location.replace("myskoda://redirect/login/#", "")

        # The last redirection starting with `myskoda://` was encountered.
        # The URL will contain the information we need as query parameters, without the leading `?`.
        data = {}
        for code in codes.split("&"):
            [key, value] = code.split("=")
            data[key] = value

        return IDKAuthorizationCode(**data)


async def _exchange_auth_code_for_idk_session(
    session: ClientSession, code: str, verifier: str
) -> IDKSession:
    """Exchange the ident login code for an auth token from Skoda.

    This will return multiple tokens, such as an access token and a refresh token.
    """
    json_data = {
        "code": code,
        "redirectUri": "myskoda://redirect/login/",
        "verifier": verifier,
    }

    async with session.post(
        f"{BASE_URL_SKODA}/api/v1/authentication/exchange-authorization-code?tokenType=CONNECT",
        json=json_data,
        allow_redirects=False,
    ) as response:
        return IDKSession(**await response.json())


async def idk_authorize(session: ClientSession, email: str, password: str) -> IDKSession:
    """Perform the full login process.

    Must be called before any other methods on the class can be called.
    """
    # Generate a random string for the OAUTH2 PKCE challenge.
    # (https://www.oauth.com/oauth2-servers/pkce/authorization-request/)
    verifier = "".join(random.choices(string.ascii_uppercase + string.digits, k=16))  # noqa: S311

    # Call the initial OIDC (OpenID Connect) authorization, giving us the initial SSO information.
    # The full flow is explain a little bit here:
    # https://openid.net/specs/openid-connect-core-1_0.html#ImplicitFlowAuth
    login_meta = await _initial_oidc_authorize(session, verifier, email, password)

    # Use the information to login with the email address,
    # which is an extra step before the actual login.
    login_meta = await _enter_email_address(session, login_meta)

    # Perform the actual login which will result in a token that can be exchanged for
    # an access token at the Skoda server.
    authentication = await _enter_password(session, login_meta)

    # Exchange the token for access and refresh tokens (JWT format).
    return await _exchange_auth_code_for_idk_session(session, authentication.code, verifier)


class AuthorizationError(Exception):
    """Error to indicate that something unexpected happened during authorization."""



"""A library for interacting with the MySkoda APIs."""

from .authorization import (
    AuthorizationError,
    IDKAuthorizationCode,
    IDKCredentials,
    IDKSession,
    idk_authorize,
)
from .models import air_conditioning, charging, common, health, info, position, status
from .rest_api import RestApi, Vehicle

__all__ = [
    "AuthorizationError",
    "IDKAuthorizationCode",
    "IDKCredentials",
    "IDKSession",
    "idk_authorize",
    "air_conditioning",
    "charging",
    "common",
    "health",
    "info",
    "position",
    "status",
    "RestApi",
    "Vehicle",
]



"""Models for responses of api/v2/air-conditioning endpoint."""

from datetime import datetime, time
from enum import StrEnum
from typing import Any

from pydantic import BaseModel, Field

from .common import ChargerLockedState, ConnectionState, OnOffState, Side, Weekday


class TemperatureUnit(StrEnum):
    CELSIUS = "CELSIUS"


class TimerMode(StrEnum):
    ONE_OFF = "ONE_OFF"
    RECURRING = "RECURRING"


class Timer(BaseModel):
    enabled: bool
    id: int
    selected_days: list[Weekday] = Field(None, alias="selectedDays")
    time: time
    type: TimerMode


class SeatHeating(BaseModel):
    front_left: bool = Field(None, alias="frontLeft")
    front_right: bool = Field(None, alias="frontRight")


class TargetTemperature(BaseModel):
    temperature_value: float = Field(None, alias="temperatureValue")
    unit_in_car: TemperatureUnit = Field(None, alias="unitInCar")


class WindowHeatingState(BaseModel):
    front: OnOffState
    rear: OnOffState
    unspecified: Any


class AirConditioning(BaseModel):
    """Information related to air conditioning."""

    air_conditioning_at_unlock: bool = Field(None, alias="airConditioningAtUnlock")
    car_captured_timestamp: datetime = Field(None, alias="carCapturedTimestamp")
    charger_connection_state: ConnectionState = Field(None, alias="chargerConnectionState")
    charger_lock_state: ChargerLockedState = Field(None, alias="chargerLockState")
    errors: list[Any]
    estimated_date_time_to_reach_target_temperature: datetime = Field(
        None, alias="estimatedDateTimeToReachTargetTemperature"
    )
    seat_heating_activated: SeatHeating = Field(None, alias="seatHeatingActivated")
    state: OnOffState
    steering_wheel_position: Side = Field(None, alias="steeringWheelPosition")
    target_temperature: TargetTemperature | None = Field(None, alias="targetTemperature")
    timers: list[Timer]
    window_heating_enabled: bool = Field(None, alias="windowHeatingEnabled")
    window_heating_state: WindowHeatingState = Field(None, alias="windowHeatingState")



"""Models for responses of api/v2/vehicle-status/{vin}/driving-range endpoint."""

from datetime import datetime
from enum import StrEnum

from pydantic import BaseModel, Field


class EngineType(StrEnum):
    DIESEL = "diesel"
    ELECTRIC = "electric"


class EngineRange(BaseModel):
    current_fuel_level_in_percent: int = Field(None, alias="currentFuelLevelInPercent")
    current_so_c_in_percent: int = Field(None, alias="currentSoCInPercent")
    engine_type: EngineType = Field(None, alias="engineType")
    remaining_range_in_km: int = Field(None, alias="remainingRangeInKm")


class DrivingRange(BaseModel):
    car_captured_timestamp: datetime = Field(None, alias="carCapturedTimestamp")
    car_type: EngineType = Field(None, alias="carType")
    primary_engine_range: EngineRange = Field(None, alias="primaryEngineRange")
    total_range_in_km: int = Field(None, alias="totalRangeInKm")



"""Models for responses of api/v2/vehicle-status/{vin}."""

from datetime import datetime

from pydantic import BaseModel, Field

from myskoda.models.common import DoorLockedState, OnOffState, OpenState


class Detail(BaseModel):
    bonnet: OpenState
    sunroof: OpenState
    trunk: OpenState


class Overall(BaseModel):
    doors: OpenState
    doors_locked: DoorLockedState = Field(None, alias="doorsLocked")
    lights: OnOffState
    locked: DoorLockedState
    windows: OpenState


class Status(BaseModel):
    """Current status information for a vehicle."""

    car_captured_timestamp: datetime = Field(None, alias="carCapturedTimestamp")
    detail: Detail
    overall: Overall



"""Models for responses of api/v1/vehicle-health-report/warning-lights endpoint."""

from datetime import datetime
from enum import StrEnum
from typing import Any

from pydantic import BaseModel, Field


class WarningLightCategory(StrEnum):
    ASSISTANCE = "ASSISTANCE"
    COMFORT = "COMFORT"
    BRAKE = "BRAKE"
    ELECTRIC_ENGINE = "ELECTRIC_ENGINE"
    ENGINE = "ENGINE"
    LIGHTING = "LIGHTING"
    TIRE = "TIRE"
    OTHER = "OTHER"


class WarningLight(BaseModel):
    category: WarningLightCategory
    defects: list[Any]


class Health(BaseModel):
    """Information about the car's health (currently only mileage)."""

    captured_at: datetime = Field(None, alias="capturedAt")
    mileage_in_km: int = Field(None, alias="mileageInKm")
    warning_lights: list[WarningLight] = Field(None, alias="warningLights")



"""Common models used in multiple responses."""

from enum import StrEnum

from pydantic import BaseModel, Field


class OnOffState(StrEnum):
    ON = "ON"
    OFF = "OFF"
    INVALID = "INVALID"


class EnabledState(StrEnum):
    ENABLED = "ENABLED"
    DISABLED = "DISABLED"


class ActiveState(StrEnum):
    ACTIVATED = "ACTIVATED"
    DEACTIVATED = "DEACTIVATED"


class OpenState(StrEnum):
    OPEN = "OPEN"
    CLOSED = "CLOSED"
    UNSUPPORTED = "UNSUPPORTED"


class DoorLockedState(StrEnum):
    LOCKED = "YES"
    UNLOCKED = "NO"


class ChargerLockedState(StrEnum):
    LOCKED = "LOCKED"
    UNLOCKED = "UNLOCKED"


class ConnectionState(StrEnum):
    CONNECTED = "CONNECTED"
    DISCONNECTED = "DISCONNECTED"


class Side(StrEnum):
    LEFT = "LEFT"
    RIGHT = "RIGHT"


class Coordinates(BaseModel):
    latitude: float
    longitude: float


class Address(BaseModel):
    city: str
    country: str | None
    country_code: str = Field(None, alias="countryCode")
    house_number: str | None = Field(None, alias="houseNumber")
    street: str
    zip_code: str = Field(None, alias="zipCode")


class Weekday(StrEnum):
    MONDAY = "MONDAY"
    TUESDAY = "TUESDAY"
    WEDNESDAY = "WEDNESDAY"
    THURSDAY = "THURSDAY"
    FRIDAY = "FRIDAY"
    SATURDAY = "SATURDAY"
    SUNDAY = "SUNDAY"



"""Models for responses of api/v2/vehicle-status/{vin}/driving-range."""

from enum import StrEnum

from pydantic import BaseModel, Field

from .common import Address, Coordinates


class PositionType(StrEnum):
    VEHICLE = "VEHICLE"


class Position(BaseModel):
    address: Address
    gps_coordinates: Coordinates = Field(None, alias="gpsCoordinates")
    type: PositionType


class ErrorType(StrEnum):
    VEHICLE_IN_MOTION = "VEHICLE_IN_MOTION"


class Error(BaseModel):
    type: ErrorType
    description: str


class Positions(BaseModel):
    """Positional information (GPS) for the vehicle and other things."""

    errors: list[Error]
    positions: list[Position]



"""Models for responses of api/v2/garage/vehicles/{vin}."""

from datetime import date
from enum import StrEnum
from typing import Any

from pydantic import BaseModel, Field


class CapabilityId(StrEnum):
    AIR_CONDITIONING = "AIR_CONDITIONING"
    AIR_CONDITIONING_SAVE_AND_ACTIVATE = "AIR_CONDITIONING_SAVE_AND_ACTIVATE"
    AIR_CONDITIONING_SMART_SETTINGS = "AIR_CONDITIONING_SMART_SETTINGS"
    AIR_CONDITIONING_TIMERS = "AIR_CONDITIONING_TIMERS"
    AUTOMATION = "AUTOMATION"
    BATTERY_CHARGING_CARE = "BATTERY_CHARGING_CARE"
    BATTERY_SUPPORT = "BATTERY_SUPPORT"
    CARE_AND_INSURANCE = "CARE_AND_INSURANCE"
    CHARGE_MODE_SELECTION = "CHARGE_MODE_SELECTION"
    CHARGING = "CHARGING"
    CHARGING_MEB = "CHARGING_MEB"
    CHARGING_PROFILES = "CHARGING_PROFILES"
    CHARGING_STATIONS = "CHARGING_STATIONS"
    CUBIC = "CUBIC"
    DEALER_APPOINTMENT = "DEALER_APPOINTMENT"
    DESTINATIONS = "DESTINATIONS"
    DESTINATION_IMPORT = "DESTINATION_IMPORT"
    DESTINATION_IMPORT_UPGRADABLE = "DESTINATION_IMPORT_UPGRADABLE"
    DIGICERT = "DIGICERT"
    EMERGENCY_CALLING = "EMERGENCY_CALLING"
    EV_ROUTE_PLANNING = "EV_ROUTE_PLANNING"
    EXTENDED_CHARGING_SETTINGS = "EXTENDED_CHARGING_SETTINGS"
    FUEL_STATUS = "FUEL_STATUS"
    GEO_FENCE = "GEO_FENCE"
    GUEST_USER_MANAGEMENT = "GUEST_USER_MANAGEMENT"
    HONK_AND_FLASH = "HONK_AND_FLASH"
    ICE_VEHICLE_RTS = "ICE_VEHICLE_RTS"
    MAP_UPDATE = "MAP_UPDATE"
    MEASUREMENTS = "MEASUREMENTS"
    MISUSE_PROTECTION = "MISUSE_PROTECTION"
    NEWS = "NEWS"
    ONLINE_SPEECH_GPS = "ONLINE_SPEECH_GPS"
    PARKING_INFORMATION = "PARKING_INFORMATION"
    PARKING_POSITION = "PARKING_POSITION"
    PAY_TO_FUEL = "PAY_TO_FUEL"
    PAY_TO_PARK = "PAY_TO_PARK"
    PLUG_AND_CHARGE = "PLUG_AND_CHARGE"
    POI_SEARCH = "POI_SEARCH"
    POWERPASS_TARIFFS = "POWERPASS_TARIFFS"
    ROADSIDE_ASSISTANT = "ROADSIDE_ASSISTANT"
    ROUTE_IMPORT = "ROUTE_IMPORT"
    ROUTE_PLANNING_5_CHARGERS = "ROUTE_PLANNING_5_CHARGERS"
    ROUTING = "ROUTING"
    SERVICE_PARTNER = "SERVICE_PARTNER"
    SPEED_ALERT = "SPEED_ALERT"
    STATE = "STATE"
    SUBSCRIPTIONS = "SUBSCRIPTIONS"
    TRAFFIC_INFORMATION = "TRAFFIC_INFORMATION"
    TRIP_STATISTICS = "TRIP_STATISTICS"
    VEHICLE_HEALTH_INSPECTION = "VEHICLE_HEALTH_INSPECTION"
    VEHICLE_HEALTH_WARNINGS = "VEHICLE_HEALTH_WARNINGS"
    VEHICLE_HEALTH_WARNINGS_WITH_WAKE_UP = "VEHICLE_HEALTH_WARNINGS_WITH_WAKE_UP"
    VEHICLE_SERVICES_BACKUPS = "VEHICLE_SERVICES_BACKUPS"
    VEHICLE_WAKE_UP = "VEHICLE_WAKE_UP"
    VEHICLE_WAKE_UP_TRIGGER = "VEHICLE_WAKE_UP_TRIGGER"
    WARNING_LIGHTS = "WARNING_LIGHTS"
    WEB_RADIO = "WEB_RADIO"
    WINDOW_HEATING = "WINDOW_HEATING"


class CapabilityStatus(StrEnum):
    DEACTIVATED_BY_ACTIVE_VEHICLE_USER = "DEACTIVATED_BY_ACTIVE_VEHICLE_USER"


class Capability(BaseModel):
    id: CapabilityId
    statuses: list[CapabilityStatus]


class Capabilities(BaseModel):
    capabilities: list[Capability]


class Battery(BaseModel):
    capacity: int = Field(None, alias="capacityInKWh")


class BodyType(StrEnum):
    SUV = "SUV"
    COMBI = "Combi"


class VehicleState(StrEnum):
    ACTIVATED = "ACTIVATED"


class Engine(BaseModel):
    power: int = Field(None, alias="powerInKW")
    capacity_in_liters: float | None = Field(None, alias="capacityInLiters")
    type: str


class Gearbox(BaseModel):
    type: str


class Specification(BaseModel):
    battery: Battery | None
    body: BodyType
    engine: Engine
    manufacturing_date: date = Field(None, alias="manufacturingDate")
    max_charging_power: int = Field(None, alias="maxChargingPowerInKW")
    model: str
    model_year: str = Field(None, alias="modelYear")
    system_code: str = Field(None, alias="systemCode")
    system_model_id: str = Field(None, alias="systemModelId")
    title: str
    trim_level: str = Field(None, alias="trimLevel")


class ServicePartner(BaseModel):
    id: str = Field(None, alias="servicePartnerId")


class ErrorType(StrEnum):
    MISSING_RENDER = "MISSING_RENDER"


class Error(BaseModel):
    description: str
    type: ErrorType


class Info(BaseModel):
    """Basic vehicle information."""

    software_version: str = Field(None, alias="softwareVersion")
    state: VehicleState
    specification: Specification
    vin: str
    name: str
    device_platform: str = Field(None, alias="devicePlatform")
    service_partner: ServicePartner = Field(None, alias="servicePartner")
    workshop_mode_enabled: bool = Field(None, alias="workshopModeEnabled")
    capabilities: Capabilities
    errors: list[Error] | None
    license_plate: str = Field(None, alias="licensePlate")



"""Models for responses of api/v3/vehicle-maintenance/vehicles/{vin} endpoint."""

from datetime import datetime, time
from enum import StrEnum

from pydantic import BaseModel, Field

from .common import Address, Coordinates, Weekday


class MaintenanceReport(BaseModel):
    captured_at: datetime = Field(None, alias="capturedAt")
    inspection_due_in_days: int = Field(None, alias="inspectionDueInDays")
    inspection_due_in_km: int = Field(None, alias="inspectionDueInKm")
    mileage_in_km: int = Field(None, alias="mileageInKm")
    oil_service_due_in_days: int = Field(None, alias="oilServiceDueInDays")
    oil_service_due_in_km: int = Field(None, alias="oilServiceDueInKm")


class Contact(BaseModel):
    email: str
    phone: str
    url: str


class TimeRange(BaseModel):
    start: time = Field(None, alias="from")
    end: time = Field(None, alias="to")


class OpeningHoursPeriod(BaseModel):
    opening_times: list[TimeRange] = Field(None, alias="openingTimes")
    period_end: Weekday = Field(None, alias="periodEnd")
    period_start: Weekday = Field(None, alias="periodStart")


class CommunicationChannel(StrEnum):
    email = "EMAIL"


class PredictiveMaintenanceSettings(BaseModel):
    email: str
    phone: str
    preferred_channel: CommunicationChannel = Field(None, alias="preferredChannel")
    service_activated: bool = Field(None, alias="serviceActivated")


class PredictiveMaintenance(BaseModel):
    setting: PredictiveMaintenanceSettings


class ServicePartner(BaseModel):
    address: Address
    brand: str
    contact: Contact
    id: str
    location: Coordinates
    name: str
    opening_hours: list[OpeningHoursPeriod] = Field(None, alias="openingHours")
    partner_number: str = Field(None, alias="partnerNumber")


class Maintenance(BaseModel):
    maintenance_report: MaintenanceReport = Field(None, alias="maintenanceReport")
    predictive_maintenance: PredictiveMaintenance | None = Field(
        None, alias="predictiveMaintenance"
    )
    preferred_service_partner: ServicePartner = Field(None, alias="preferredServicePartner")



"""Models for responses of api/v2/vehicle-status/{vin}."""

from datetime import date
from enum import StrEnum

from pydantic import BaseModel, Field


class VehicleType(StrEnum):
    FUEL = "FUEL"


class StatisticsEntry(BaseModel):
    date: date
    average_fuel_consumption: float | None = Field(None, alias="averageFuelConsumption")
    average_speed_in_kmph: int | None = Field(None, alias="averageSpeedInKmph")
    mileage_in_km: int | None = Field(None, alias="mileageInKm")
    travel_time_in_min: int | None = Field(None, alias="travelTimeInMin")
    trip_ids: list[int] | None = Field(None, alias="tripIds")


class TripStatistics(BaseModel):
    overall_average_fuel_consumption: float = Field(None, alias="overallAverageFuelConsumption")
    overall_average_mileage_in_km: int = Field(None, alias="overallAverageMileageInKm")
    overall_average_speed_in_kmph: int = Field(None, alias="overallAverageSpeedInKmph")
    overall_average_travel_time_in_min: int = Field(None, alias="overallAverageTravelTimeInMin")
    overall_mileage_in_km: int = Field(None, alias="overallMileageInKm")
    overall_travel_time_in_min: int = Field(None, alias="overallTravelTimeInMin")
    vehicle_type: VehicleType = Field(None, alias="vehicleType")
    detailed_statistics: list[StatisticsEntry] = Field(None, alias="detailedStatistics")
